// Alathon's simple MySQL library: simpleMySQL.dm
var
	const
		TYPE_NONE   = 0
		TYPE_EXPIRE = 1
		TYPE_LIST   = 2
		TYPE_DEFAULT  = TYPE_EXPIRE
		DBQUERY_TIMEOUT = 600// 60s timeout on non-locked queries.
		DBQUERY_BUSY_INTERVAL = 20

DBQuery
	proc
		LockQuery()
		Used()

	var
		busy = 0
		expires
		locked = 0

	Used()
		expires = (mysql.mysql_type == TYPE_EXPIRE ? world.realtime + DBQUERY_TIMEOUT : world.realtime + DBQUERY_BUSY_INTERVAL)

	GetRowData()
		. = ..()
		Used()

	NextRow()
		. = ..()
		Used()

	Execute()
		. = ..()
		Used()

var/mysql_connection/mysql
mysql_connection
	var
		mysql_type = TYPE_DEFAULT
		default_queries_size = 15
		maximum_queries_size = 25
		DBConnection/dbcon
		list/queries

	proc
		CreateQuery()
		CleanQueries()
		SetupQueries()

	New()
		dbcon = new()

	CleanQueries()
		set background = 1
		spawn() while(1)
			if(queries) for(var/DBQuery/Q in queries)
				if(world.realtime >= Q.expires && !Q.locked)
					if(mysql_type == TYPE_EXPIRE)
						del Q
						if(!queries.len)
							queries = null
					else if(mysql_type == TYPE_LIST)
						Q.Close()
						Q.busy = 0
			sleep(mysql_type == TYPE_EXPIRE ? DBQUERY_TIMEOUT : DBQUERY_BUSY_INTERVAL)

	SetupQueries()
		queries = new()
		for(var/i = 1, i <= default_queries_size, i++)
			var/DBQuery/Q = dbcon.NewQuery()
			queries += Q

	CreateQuery()
		if(!dbcon.IsConnected())
			var/ok = dbcon.Connect(DBI(), DBUser(), DBPass())
			if(!ok)
				return "Unable to initialize database connection ([dbcon.ErrorMsg()])"
		var/DBQuery/Q

		if(mysql_type == TYPE_EXPIRE)
			Q = dbcon.NewQuery()
			if(!queries) queries = new()
			queries += Q
			return Q

		else if(mysql_type == TYPE_NONE)
			Q = dbcon.NewQuery()
			return Q

		else if(mysql_type == TYPE_LIST)
			if(!queries)
				SetupQueries()

			. = 0
			for(Q in queries)
				if(!Q.busy)
					. = 1
					Q.busy = 1
					break

			if(!.)
				if(length(queries) >= maximum_queries_size) // No more!
					return "Unable to create new query: Maximum query size list reached"

				Q = dbcon.NewQuery()
				queries += Q
			return Q



// Alathon's simple MySQL library: sqlformat.dm
proc
	SQLFormat_Insert(_tbl, list/L, Replace=1)
		if(!L || !istype(L, /list))
			return 0

		var/i = 1
		var/list_len = length(L)

		. = "[Replace?"REPLACE":"INSERT"] INTO `[_tbl]` ("
		for(i, i <= list_len, i++)
			. += "`[L[i]]`[i == list_len ? "":","]"
		. += ") VALUES ("
		i = 1
		for(i, i <= list_len, i++)
			var/j = L[i]
			var/value = L[j]
			. += "'[value]'[i == list_len ? "":","]"
		. += ")"

	SQLFormat_Select(_tbl, what = "*", cond = "WHERE 1")
		if(istext(what))
			. = "SELECT [what] FROM `[_tbl]` [cond]"
		else if(istype(what, /list))
			. = "SELECT "
			var/list/L = what
			var/list_len = length(L)
			for(var/i = 1, i <= list_len, i++)
				var/v = L[i]
				. += "[v][i == list_len ? "":","]"
			. += " FROM [_tbl] [cond]"

	SQLFormat_Update(_tbl, list/what, cond = "WHERE 1")
		if(istext(what))
			. = "UPDATE `[_tbl]` SET `[what]` WHERE [cond]"
		else if(istype(what, /list))
			. = "UPDATE `[_tbl]` SET "
			var/list_len = length(what)
			for(var/i = 1, i <= list_len, i++)
				var/v = what[i]
				. += "`[v]` = '[what[v]]'[i == list_len ? "":","]"
			. += " [cond]"

	SQLFormat_Delete(_tbl, cond = "WHERE 1")
		return "DELETE FROM `[_tbl]` [cond]"

	SQLFormat_SelectLastID(_tbl)
		return "SELECT LAST_INSERT_ID() FROM `[_tbl]`"